@@ -5,6 +5,6 @@ Jupiterweb API
 """
 
 __author__ = 'Erick Ghuron'
-__version__ = '1.3.2'
+__version__ = '1.4.2'
 
-from .api import JupiterWeb
+from jupiterweb.api import JupiterWeb
